// ReCo Remote Contact Receiver (ATTiny2313 8MHz)
// V2 - Copyleft furrtek 2012

#define BUZZER PD1
#define RADIO PD2
#define RELAY PD3
#define LED_GREEN PD4
#define LED_RED PD5

#define TIMECONST 1500

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

volatile uint8_t armed=0;
volatile uint32_t word=0;
uint8_t relais;

ISR(TIMER0_OVF_vect) {
	word = 0;				// Timeout
}

ISR(INT0_vect) {
	cli();
	if (bit_is_set(PIND,RADIO)) {	// Front montant
		PORTD |= _BV(LED_RED);
		TCNT0 = 0;				// Reset timer
		TCCR0A = 0;
		TCCR0B = 0b00000100; 	// Prescaler  256 (timeout 8ms)
	} else {					// Front descendant
		PORTD &= ~_BV(LED_RED);
		word <<= 1;
		if (TCNT0 >= 64) word |= 1;
		if (word == 0x0FEE1BAD) {		// Octet d'armement ?
			PORTD |= _BV(LED_GREEN); 	// LED verte allume
			armed = 1;					// Arm OK.
		}
		if ((word == 0xDEADBEEF) && armed) {// Octet de dtonnation
			PORTD |= _BV(BUZZER);
			for (relais=0;relais<3;relais++) {
				PORTD |= _BV(RELAY) | _BV(LED_GREEN);
				_delay_ms(300);
				PORTD &= ~_BV(RELAY);
				_delay_ms(1000);
			}
			armed = 0;
			PORTD &= ~(_BV(LED_GREEN) | _BV(BUZZER));
		}
		TCCR0B = 0; 	// Timer0 off
	}
}

int main(void) {
	WDTCSR |= (1<<WDCE) | (1<<WDE);	// Watchdog off
	WDTCSR = 0x00;

	DDRB = 0x00;
    DDRD = _BV(BUZZER) | _BV(LED_RED) | _BV(LED_GREEN) | _BV(RELAY);
	TIMSK = 0b00000010; 	// OVF

	PORTD = _BV(LED_RED) | _BV(LED_GREEN);	// Hello !
	_delay_ms(1000);
	PORTD = 0;

	MCUCR = 0b00000001; 	// INT0 (PD2) change
	GIMSK = 0b01000000;		// INT0
	sei();

	while (1) {
		if (armed == 1) {
			PORTD |= _BV(BUZZER);
			_delay_ms(30);
			PORTD &= ~_BV(BUZZER);
			_delay_ms(100);
		} else {
			PORTD &= ~_BV(BUZZER);
		}
	}
	return 0;
}
